-- $Id: bwaa-analysis.ads 95 2009-06-05 14:49:48Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

with Ast2Cfg.Cfgs;
with Ast2Cfg.Nodes;

with Bwaa.Graph;
with Bwaa.Analysis_Types; use Bwaa.Analysis_Types;
with Bwaa.Analysis_Types.Variable_Set; use Bwaa.Analysis_Types.Variable_Set;
with Bwaa.Util; use Bwaa.Util;
with Bwaa.Aliasing;
with Bwaa.Visibility; use Bwaa.Visibility;
with Bwaa.Visibility.Visibility_Info;

package Bwaa.Analysis is

   procedure Analyse (Ast2Cfg_Cfg: in Ast2Cfg.Cfgs.Cfg_Object;
                      Specs: in Specification_Map.Map;
                      Visi_Info: in Visibility_Info.Info);

   Analysis_Exception: exception;

private

   Aliases: aliased Aliasing.Alias_Info;

   procedure Find_Termination_Statements (Bwaa_Cfg: in out Bwaa.Graph.Cfg);

   -- check whether the variable or an equivalent name is already in the set
   function Already_In_Variable_Set (S: in Variable_Set.Set; Name: in Unbounded_String) return Boolean;

   procedure Harvest_Param_Tree (Cur: in Ast2Cfg.Nodes.Node_Class_Ptr; Variables: in out Variable_Set.Set);

   procedure Find_Busy_Wait_Candidates (Bwaa_Cfg: in out Bwaa.Graph.Cfg);

   procedure Find_Indirect_Candidates (Bwaa_Cfg: in out Bwaa.Graph.Cfg);

   function Defines (Node: in Ast2Cfg.Nodes.Node_Class_Ptr; Var: in Variable) return Boolean;

   procedure Busy_Var_Check (Bwaa_Cfg: in Bwaa.Graph.Cfg;
                             L00p: in Bwaa.Graph.L00p;
                             Termination_Statement: in Ast2Cfg.Long_Long_Natural;
                             BW_Candidate: in out Variable);

   procedure Find_Busy_Wait_Variables (Bwaa_Cfg: in out Bwaa.Graph.Cfg);

end Bwaa.Analysis;
