-- $Id: bwaa-analysis_types.adb 61 2009-04-05 18:27:40Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Strings.Unbounded.Hash;

package body Bwaa.Analysis_Types is

   procedure Init(V: in out Variable; Name: in Unbounded_String) is
   begin
      V.Name := Name;
   end Init;

   procedure Init(V: in out Variable; Name: in String) is
   begin
      V.Name := S2u(Name);
   end Init;

   function "=" (Left: in Variable; Right: in Variable) return Boolean is
   begin
      if Left.Name = Right.Name then
         return True;
      end if;

      return False;
   end "=";

   function Get_Name(V: in Variable) return Unbounded_String is
   begin
      return V.Name;
   end Get_Name;

   function Get_Name(V: in Variable) return String is
   begin
      return U2s(V.Name);
   end Get_Name;

   procedure Set_Busy_Var(V: in out Variable) is
   begin
      V.Busy_Var := True;
   end Set_Busy_Var;

   function Is_Busy_Var(V: in Variable) return Boolean is
   begin
      return V.Busy_Var;
   end Is_Busy_Var;

   procedure Set_Direct_Candidate_Name(V: in out Variable;
                                     Direct_Candidate_Name: in Unbounded_String) is
   begin
      V.Direct_Candidate_Name := Direct_Candidate_Name;
   end Set_Direct_Candidate_Name;

   procedure Set_Direct_Candidate_Name(V: in out Variable;
                                     Direct_Candidate_Name: in String) is
   begin
      V.Direct_Candidate_Name := S2u(Direct_Candidate_Name);
   end Set_Direct_Candidate_Name;

   function Get_Direct_Candidate_Name(V: in Variable) return Unbounded_String is
   begin
      return V.Direct_Candidate_Name;
   end Get_Direct_Candidate_Name;

   function Get_Direct_Candidate_Name(V: in Variable) return String is
   begin
      return U2s(V.Direct_Candidate_Name);
   end Get_Direct_Candidate_Name;

   function Hash (V: in Variable) return Ada.Containers.Hash_Type is
   begin
      return Ada.Strings.Unbounded.Hash(V.Name);
   end Hash;

end Bwaa.Analysis_Types;
