-- $Id: bwaa-tarjan.ads 59 2009-04-05 10:37:07Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ast2Cfg;

with Bwaa.Union_Find;
with Bwaa.Graph; use Bwaa.Graph;
use Bwaa;

generic
   Size: in Ast2Cfg.Long_Long_Natural;
package Bwaa.Tarjan is

   -- construct the loop forest (Cfg.L_Forest) using a slightly modified version of Tarjans algorithm
   procedure Build_Loop_Forest (Cfg: in out Graph.Cfg);

private

   package Uf is new Union_Find(Ast2Cfg.Long_Long_Natural, 1, Size);

   Uf_Sets: Uf.Sets;
   Loop_Count: Ast2Cfg.Long_Long_Natural := 0;
   Header_To_Loop: Vertex_Map.Map; -- maps the dfs number of the loop header to the loop number

end Bwaa.Tarjan;
