-- $Id: bwaa-traversal.ads 95 2009-06-05 14:49:48Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

with Ast2Cfg.Flow_Types;
with Ast2Cfg.Cfgs;

with Bwaa.Graph;
with Bwaa.Visibility.Visibility_Info; use Bwaa.Visibility.Visibility_Info;
use Bwaa.Visibility;
with Bwaa.Util; use Bwaa.Util;

package Bwaa.Traversal is

   -- Fill Bwaa_Cfg.Dfs_Sorted and Bwaa_Cfg.Id_Dfs
   procedure Dfs (Ast2Cfg_Cfg: in Ast2Cfg.Cfgs.CFG_Object; Bwaa_Cfg: out Bwaa.Graph.Cfg);

   -- Find all specifications and fill a hashed map with references to them
   procedure Collect_Specs (Flow: in Ast2Cfg.Flow_Types.Flow_Class_Ptr;
                            Specs: in out Specification_Map.Map;
                            Path: in Unbounded_String);

   -- collect the variables, packages and renamings that are visible in each flow
   procedure Build_Visibility_Info (Flow: in Ast2Cfg.Flow_Types.Flow_Class_Ptr;
                                   Visi_Info: in out Visibility_Info.Info;
                                   Passed_On: in Visible_Set.Set;
                                   Pkg_Passed_On: in Visible_Set.Set;
                                   Spec_Map: in Bwaa.Util.Specification_Map.Map);

private

   Node_Count: Ast2Cfg.Long_Long_Natural := 0;

end Bwaa.Traversal;
