-- $Id: bwaa-union_find.ads 59 2009-04-05 10:37:07Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

generic

   type Element is (<>);
   First: in Element;
   Last: in Element;

-- the classical union find data structure with path compression
-- (union-by-rank is not applicable for this program)
package Bwaa.Union_Find is

   type Sets is private;

   Out_Of_Range: exception;

   -- create a set containing only X
   procedure Makeset (S: in out Sets; X: in Element);

   -- call Makeset for all elements
   procedure Init_Sets (S: in out Sets);

   -- finds the canonical element for the set containing X; uses path compression
   procedure Find (S: in out Sets; X: in Element; Canonical: out Element);

   -- links X to Y, so that find on X and all its descendants returns
   -- the canonical element of the set containing Y
   procedure Link (S: in out Sets; X: in Element; Y: in Element);

   -- links the canonical elements of the sets containing X or Y
   procedure Union (S: in out Sets; X: in Element; Y: in Element);

   procedure Output(S: in Sets); -- TODO: remove

private

   type Sets is array (First .. Last) of Element;

end Bwaa.Union_Find;
