-- $Id: bwaa-util.ads 97 2009-06-05 20:01:48Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Hash;
with Ada.Containers.Hashed_Maps;
with Ada.Containers.Hashed_Sets;

with Ast2Cfg;
use type Ast2Cfg.Long_Long_Natural;
with Ast2Cfg.Flow_Types;
use type Ast2Cfg.Flow_Types.Flow_Class_Ptr;
with Ast2Cfg.Id;

package Bwaa.Util is

   type Unbounded_String_Array is array (Natural range <>) of Unbounded_String;

   package Id_To_Dfs_Num is new Ada.Containers.Hashed_Maps (Key_Type => Ast2Cfg.Id.Pointer,
                                                            Element_Type => Ast2Cfg.Long_Long_Natural,
                                                            Hash => Ast2Cfg.Id.Hash,
                                                            Equivalent_Keys => Ast2Cfg.Id."=");

   package Specification_Map is new Ada.Containers.Hashed_Maps (Key_Type => Unbounded_String,
                                                                Element_Type => Ast2Cfg.Flow_Types.Flow_Class_Ptr,
                                                                Hash => Ada.Strings.Unbounded.Hash,
                                                                Equivalent_Keys => Ada.Strings.Unbounded."=");

   package Unbounded_String_Set is new Ada.Containers.Hashed_Sets (Element_Type => Unbounded_String,
                                                                   Hash => Ada.Strings.Unbounded.Hash,
                                                                   Equivalent_Elements => Ada.Strings.Unbounded."=");

   function Hash (X: in Ast2Cfg.Long_Long_Natural) return Ada.Containers.Hash_Type;

   -- Convert a String to an Unbounded String
   function S2u(Str: in String) return Unbounded_String;

   -- Convert an Unbounded_String to a String
   function U2s(U_Str: in Unbounded_String) return String;

   -- Convert a Wide_String to a String
   function W2s(W_Str: in Wide_String) return String;

   -- Split an Unbounded_String into multiple Unbounded_Strings based on a separator
   function Split(U: in Unbounded_String; Separator: in String := ".") return Unbounded_String_Array;

   -- Merge multiple Unbounded_Strings into a single Unbounded_String using a separator
   function Merge(U: in Unbounded_String_Array;
                  Stop: in Positive;
                  Start: in Positive := 1;
                  Separator: in String := ".") return Unbounded_String;

   -- Merge multiple Unbounded_Strings into a single Unbounded_String using a separator
   function Merge(U: in Unbounded_String_Array) return Unbounded_String;

end Bwaa.Util;
