-- $Id: bwaa-visibility.adb 92 2009-06-03 08:35:28Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ast2Cfg.Declaration_Id; use Ast2Cfg.Declaration_Id;

package body Bwaa.Visibility is

   procedure Init (Decl: in out Declaration;
                   Name: in Unbounded_String;
                   Prefix: in Unbounded_String;
                   Id: in Ast2Cfg.Declaration_Id.Object) is
   begin
      Decl.Name := Name;
      Decl.Prefix := Prefix;

      if Id = Null_Declaration_Id then
         Decl.Id := Id;
         raise Visibility_Exception with "Null_Declaration_Id encountered.";
      end if;

      Decl.Id := Id;
   end Init;

   function "=" (Left: in Declaration; Right: in Declaration) return Boolean is
   begin

      --  if Left.Name = Right.Name and then
--           Left.Prefix = Right.Prefix and then
--           Left.Id = Right.Id then

      if Left.Name = Right.Name then
         return True;
      end if;

      return False;

   end "=";

   function Before (Left: in Declaration; Right: in Declaration) return Boolean is
   begin
      return Left.Id < Right.Id;
   end Before;

   function After (Left: in Declaration; Right: in Declaration) return Boolean is
   begin
      return not Before(Left, Right);
   end After;

   function Before (Left: in Declaration; Right: in Ast2Cfg.Declaration_Id.Object) return Boolean is
   begin
      return Left.Id < Right;
   end Before;

   function Before (Left: in Ast2Cfg.Declaration_Id.Object; Right: in Declaration) return Boolean is
   begin
      return Left < Right.Id;
   end Before;

   function After (Left: in Declaration; Right: in Ast2Cfg.Declaration_Id.Object) return Boolean is
   begin
      return Left.Id > Right;
   end After;

   function After (Left: in Ast2Cfg.Declaration_Id.Object; Right: in Declaration) return Boolean is
   begin
      return Left > Right.Id;
   end After;

   function Get_Name (Decl: in Declaration) return Unbounded_String is
   begin
      return Decl.Name;
   end Get_Name;

   function Get_Prefix (Decl: in Declaration) return Unbounded_String is
   begin
      return Decl.Prefix;
   end Get_Prefix;

   function Get_Full_Name (Decl: in Declaration) return Unbounded_String is
   begin
      if Decl.Prefix /= Null_Unbounded_String then
         return  Decl.Prefix & "." & Decl.Name;
      else
         return Decl.Name;
      end if;
   end Get_Full_Name;

   function Get_Id (Decl: in Declaration) return Ast2Cfg.Declaration_Id.Object is
   begin
      return Decl.Id;
   end Get_Id;

   function Hash (Decl: in Declaration) return Ada.Containers.Hash_Type is
   begin
      return Ada.Strings.Unbounded.Hash(Decl.Name);
   end Hash;

end Bwaa.Visibility;
